import difflib
import json
import random
import os
import shutil
import string

from pprint import pprint

import ansigenome.utils as utils


CONFIG_TEMPLATE = """---

author_company: Acme Inc.
author_email: test@user.com
author_name: Test User
author_twitter: testuser
author_url: http://testuser

license_type: MIT
license_url: https://tldrlegal.com/license/mit-license

options_readme_template: ''
options_travis: true
options_quiet: false
options_test_runner: http://foo

default_format_gendoc: rst
default_format_graph: dot
default_format_reqs: txt
default_format_dump: json

scm_host: https://github.com
scm_repo_prefix: ansible-
scm_type: git
scm_user: testuser

graphviz_size: 15,8
graphviz_dpi: 130
graphviz_flags: ''
"""

DEFAULTS_TEMPLATE = """---
foo: bar
list:
  one: 1
  two: 2
  three: 3



# comment
baz: qux
"""

TASKS_TEMPLATE = """---
- set_fact:
  unix_is_cool: "who | grep -i blonde | date; cd ~; unzip; touch"

- set_almost_fact:
  command: "echo 'nope, not quite'"

- set_fact:
  duplicate: "not yet"

- set_fact:
  duplicate: "this should not be counted"
"""

META_TEMPLATE = """---
dependencies: []
galaxy_info: {}
ansigenome_info: {}
"""

META_TEMPLATE_FULL = """---

dependencies:
  - role: none.at_all
  - role: tear.drinker

galaxy_info:
  author: 'Chuck Norris'
  description: 'Just reading this makes you immune to death'
  company: 'Impossible'
  license: 'Death by chest hair inhalation if one does not agree'
  min_ansible_version: 'Everywhere and anything'
  platforms:
    - name: TheUniverse
  categories: [ deadly ]

ansigenome_info:
  galaxy_id: "infinity"

  travis: True

  license_url: 'There are no protocols that can withstand this license'

  authors:
    - name: 'Chuck Norris'
      url: "Ask me again for this and I'll penetrate your face with my fists"
      email: "I don't do e-mail, my mustache communicates via telekinesis"
      twitter: ''

  usage: |
    You don't use this role, it imposes itself on everyone and everything.

  custom: |
    There was a day where I once destroyed a country just by pooping.
"""


def rmrf(path):
    """
    Delete an entire path without warning.
    """
    shutil.rmtree(path)


def random_string(length=8):
    """
    Generate a random string.
    """
    return "".join(random.choice(
        string.ascii_lowercase) for _ in range(length))


def create_roles(path, number=2):
    """
    Create a number of roles.
    """
    role_names = []

    for i in range(number):
        full_path = os.path.join(path, random_string())

        utils.capture_shell("ansigenome init {0} -c system".format(full_path))

        role_names.append(os.path.basename(full_path))

    return role_names


def create_ansigenome_config(test_path):
    """
    Create a config file to use for each test.
    """
    utils.string_to_file(test_path, CONFIG_TEMPLATE)


def json_to_dict(input):
    """
    Return a dict from json.
    """
    return json.loads(input)


def print_out(title, out):
    """
    Print the output with borders.
    """
    print
    print "".join("=" * 79)
    print title
    print "".join("=" * 79)
    print out
    print "".join("=" * 79)


def populate_dict_with_files(test_path, roles, the_dict, path, tag=""):
    """
    Populate the dict to store the contents of each role's file.
    """
    print
    for role in roles:
        role_path = os.path.join(test_path, role)

        the_dict[role + tag] = utils.file_to_string(os.path.join(
            role_path, path)).splitlines(1)


def run_diff_on(the_dict, file, tag=""):
    """
    Print a context diff on 2 strings.
    """
    diff = []

    for key, values in the_dict.iteritems():
        if tag not in key:
            diff = list(difflib.context_diff(the_dict[key],
                        the_dict[key + tag]))
            print "DIFF for {0}'s {1}:".format(key, file)
            if len(diff) > 0:
                pprint(diff)
            else:
                print "[no differences]"
            print



import color as clr
import constants as c


def log(color, *args):
    """
    Print a message with a specific color.
    """
    if color in c.LOG_COLOR.keys():
        out_color = c.LOG_COLOR[color]
    else:
        out_color = color

    for arg in args:
        print clr.stringc(arg, out_color)


def ok(*args):
    """
    Print a message in green.
    """
    log("green", *args)


def warn(*args):
    """
    Print a message in yellow.
    """
    log("yellow", *args)


def error(*args):
    """
    Print a message in red.
    """
    log("red", *args)


def usage():
    """
    Return the usage for the help command.
    """
    l_bracket = clr.stringc("[", "dark gray")
    r_bracket = clr.stringc("]", "dark gray")
    pipe = clr.stringc("|", "dark gray")

    app_name = clr.stringc("%prog", "bright blue")
    commands = clr.stringc("{0}".format(pipe).join(c.VALID_ACTIONS), "normal")
    help = clr.stringc("--help", "green")
    options = clr.stringc("options", "yellow")

    guide = "\n\n"
    for action in c.VALID_ACTIONS:
        guide += command_name(app_name, action,
                              c.MESSAGES["help_" + action])

    # remove the last line break
    guide = guide[:-1]

    return "{0} {1}{2}{3} {1}{4}{3} {1}{5}{3}\n{6}".format(app_name,
                                                           l_bracket,
                                                           commands,
                                                           r_bracket,
                                                           help,
                                                           options,
                                                           guide)


def epilogue(app_name):
    """
    Return the epilogue for the help command.
    """
    app_name = clr.stringc(app_name, "bright blue")
    command = clr.stringc("command", "cyan")
    help = clr.stringc("--help", "green")

    return "\n%s %s %s for more info on a command\n" % (app_name,
                                                        command, help)


def command_name(app_name, command, help_text):
    """
    Return a snippet of help text for this command.
    """
    command = clr.stringc(command, "cyan")
    help = clr.stringc("--help", "green")

    return "{0} {1} {2}\n{3}\n\n".format(app_name, command,
                                         help, help_text)


def role(name, report, role_name_length):
    """
    Print the role information.
    """
    pad_role_name_by = 11 + role_name_length

    defaults = field_value(report["total_defaults"], "defaults", "blue", 16)
    facts = field_value(report["total_facts"], "facts", "purple", 16)
    files = field_value(report["total_files"], "files", "dark gray", 16)
    lines = field_value(report["total_lines"], "lines", "normal", 16)

    print "{0:<{1}} {2} {3} {4} {5}".format(
        clr.stringc(name, c.LOG_COLOR[report["state"]]),
        pad_role_name_by, defaults, facts, files, lines)


def field_value(key, label, color, padding):
    """
    Print a specific field's stats.
    """
    if not clr.has_colors and padding > 0:
        padding = 7

    if color == "bright gray" or color == "dark gray":
        bright_prefix = ""
    else:
        bright_prefix = "bright "

    field = clr.stringc(key, "{0}{1}".format(bright_prefix, color))
    field_label = clr.stringc(label, color)

    return "{0:>{1}} {2}".format(field, padding, field_label)


def totals(report, total_roles, role_name_length):
    """
    Print the totals for each role's stats.
    """
    roles_len_string = len(str(total_roles))
    roles_label_len = 6  # "r" "o" "l" "e" "s" " "

    if clr.has_colors:
        roles_count_offset = 22
    else:
        roles_count_offset = 13

    roles_count_offset += role_name_length

    # no idea honestly but it fixes the formatting
    # it will probably break the formatting if you have 100+ roles
    if roles_len_string > 1:
        roles_count_offset -= 2

    pad_roles_by = roles_count_offset + roles_len_string + roles_label_len

    roles = field_value(total_roles, "roles", "normal", 0)
    defaults = field_value(report["defaults"], "defaults", "blue", 16)
    facts = field_value(report["facts"], "facts", "purple", 16)
    files = field_value(report["files"], "files", "dark gray", 16)
    lines = field_value(report["lines"], "lines", "normal", 16)

    print "".join(clr.stringc("-", "black") * 79)
    print "{0} {2:>{1}} {3} {4} {5}".format(
        roles, pad_roles_by, defaults, facts, files, lines)


def gen_totals(report, file_type):
    """
    Print the gen totals.
    """
    label = clr.stringc(file_type + " files   ", "bright purple")

    ok = field_value(report["ok_role"], "ok", c.LOG_COLOR["ok"], 0)
    skipped = field_value(report["skipped_role"], "skipped",
                          c.LOG_COLOR["skipped"], 16)
    changed = field_value(report["changed_role"], "changed",
                          c.LOG_COLOR["changed"], 16)

    # missing_meta = field_value(report["missing_meta_role"],
    #                            "missing meta(s)",
    #                            c.LOG_COLOR["missing_meta"], 16)

    # print "\n{0} {1} {2} {3}".format(ok, skipped, changed, missing_meta)

    print "\n{0} {1} {2} {3}".format(label, ok, skipped, changed)


def scan_totals(report):
    """
    Print the scan totals.
    """
    ok = field_value(report["ok_role"], "ok", c.LOG_COLOR["ok"], 0)
    missing_readme = field_value(report["missing_readme_role"],
                                 "missing readme(s)",
                                 c.LOG_COLOR["missing_readme"], 16)

    missing_meta = field_value(report["missing_meta_role"],
                               "missing meta(s)",
                               c.LOG_COLOR["missing_meta"], 16)

    print "\n{0} {1} {2}".format(ok, missing_readme, missing_meta)



import os

import constants as c
import ui as ui
import utils as utils


class Run(object):
    def __init__(self, args, options, config):
        self.roles_path = args[0]
        self.options = options
        self.config = config
        self.command = options.command

        self.execute_command()

    def execute_command(self):
        """
        Execute the shell command.
        """
        stderr = ""
        role_count = 0
        for role in utils.roles_dict(self.roles_path):
            self.command = self.command.replace("%role_name", role)
            (_, err) = utils.capture_shell("cd {0} && {1}".
                                           format(os.path.join(
                                                  self.roles_path, role),
                                                  self.command))

            stderr = err
            role_count += 1

        utils.exit_if_no_roles(role_count, self.roles_path)

        if len(stderr) > 0:
            ui.error(c.MESSAGES["run_error"], stderr[:-1])
        else:
            if not self.config["options_quiet"]:
                ui.ok(c.MESSAGES["run_success"].replace(
                    "%role_count", str(role_count)), self.options.command)
